#!/usr/bin/perl
#
# cons-test [-x cons] [test_script ...]
#
#	Run the cons regression test suite.
#

require "getopts.pl";

($My_Name = $0) =~ s(.*/)();	# basename
$Usage = "Usage:  $My_Name [-q] [-d dir] [-x script] [test_script ...]
	-d dir		search for tests in specified dir
	-q		quiet, supress warnings about undefined variables
	-x script	test specified script instead of default 'cons'";

&Getopts('d:qx:') || die "$Usage\n";

$dir = (defined($opt_d)) ? $opt_d : '.';

sub env_warn {
    #
    # Make sure that platforms that don't have cc in the regular path
    # can define it. Otherwise, define it as standard 'cc'.
    # 
    if (! defined($ENV{'CC'})) {
	warn "$My_Name:  CC not defined! using 'cc'\n";
	$ENV{'CC'} = 'cc';
    }
    
    if (! defined($ENV{'AR'})) {
	warn "$My_Name:  AR not defined! using 'ar'\n";
	$ENV{'AR'} = 'ar';
    }
    
    if (! defined($ENV{'RANLIB'})) {
	warn "$My_Name:  RANLIB not defined! using 'ranlib'\n";
	$ENV{'RANLIB'} = 'ranlib';
    }
}

if (defined($opt_x)) {
    $Cons = $ENV{'PROG'} = $opt_x;
    if (! -f $Cons) {
	print STDERR "$My_Name:  The specified '$Cons' script does not exist.\n";
	print STDERR "\tCreate it, or use -x to specify some other executable script.\n";
	print STDERR "$Usage\n";
	exit 1;
    } elsif (! -x $Cons) {
	print STDERR "$My_Name:  The specified '$Cons' script is not executable.\n";
	print STDERR "\tSet execute permissions on it,\n\tor use -x to specify some other executable script.\n";
	print STDERR "$Usage\n";
	exit 1;
    }
} else {
    $Cons = 'cons';
    if (! -f $Cons) {
	print STDERR "$My_Name:  There is no executable '$Cons' script in the current directory.\n";
	print STDERR "\tCreate one, or use -x to specify some other executable script.\n";
	print STDERR "$Usage\n";
	exit 1;
    } elsif (! -x $Cons) {
	print STDERR "$My_Name:  The '$Cons' script in the local directory is not executable.\n";
	print STDERR "\tSet execute permissions on it,\n\tor use -x to specify some other executable script.\n";
	print STDERR "$Usage\n";
	exit 1;
    }
}

my $pass = 0;
my @fail;

sub report {
    my($sofar) = @_;
    if (@fail == 0) {
	print "$My_Name:  '$Cons' passed all $pass tests$sofar.\n";
    } else {
	printf "$My_Name:  '$Cons' passed $pass tests, failed %d$sofar:\n",
	scalar @fail;
	print "\t\t", join("\n\t\t", @fail), "\n";
    }
}

my $child_pid;

sub handler {
    my($sig) = @_;
    waitpid($child_pid, 0);
    print "$My_Name:  Caught SIG$sig; exiting.\n";
    print "\n";
    &report(' so far');
    exit (1);
}

$SIG{'HUP'} = \&handler;
$SIG{'INT'} = \&handler;
$SIG{'QUIT'} = \&handler;
$SIG{'TERM'} = \&handler;

sub fetch_tests {
    opendir(DIR, $_[0]) || die "$My_Name: cannot open '$_[0]': $!\n";
    my(@list) = sort grep(/^t[0-9]+a.sh$/, readdir(DIR));
    closedir(DIR);
    return @list;
}

@ARGV = &fetch_tests($dir) if ! @ARGV;

$| = 1;    	# flush print

$prefix = ($dir eq '.') ? '' : "$dir/";

$first = 1;

while (@ARGV) {
    $test = shift @ARGV;
    if ($test =~ m/([^=]*)=(.*)/o) {
	$ENV{$1} = $2;
	if (! @ARGV && @fail == 0 && $pass == 0) {
	    @ARGV = &fetch_tests($dir);
	}
	next;
    }
    if ($first) {
	&env_warn unless $opt_q;
	$first = undef;
    }
    my($cmd) = "sh $prefix$test";
    print "$My_Name:  $cmd\n";
    $child_pid = fork;
    if ($child_pid == 0) {
	exec $cmd;
    }
    waitpid($child_pid, 0);
    $exit = $?;
    if ($exit) {
	push(@fail, $test);
    } else {
	$pass++;
    }
}

print "\n";
&report();

exit @fail;
