#!/bin/sh
#
#	Compile a single executable from three .c files in a source
#	subdirectory.  One module has a conditional #include for
#	the selected OS.  Compilation takes place in two separate
#	build subdirectories established via the 'Link' command in
#	two separate invocations of cons.  The appropriate build
#	directory is specified via "OS=" on the cons command line.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [single-environment variant builds]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir src
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
die qq(OS must be specified) unless \$OS = \$ARG{OS};
\$BUILD = "#build/\$OS";
%cflags = (
	'foo'	=> '-DFOO',
	'bar'	=> '-DBAR',
);
\$env = new cons (
	CC	=> '$CC',
	CFLAGS	=> \$cflags{\$OS},
);
Export ( env );
Link \$BUILD => 'src';
Build (
	"\$BUILD/Conscript"
);
_EOF_
cat <<'_EOF_' >src/Conscript
Import ( env );
Program $env 'xxx', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/string.h
#ifdef	FOO
#define	STRING	"FOO"
#endif
#ifdef	BAR
#define	STRING	"BAR"
#endif
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/a.c
#include "string.h"
a()
{
	printf("In function a():  %s.\n", STRING);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/b.c
#include "string.h"
b()
{
	printf("In function b():  %s.\n", STRING);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/main.c
#include "string.h"
main()
{
#ifdef	BAR
	printf("Only when BAR:\n");
#endif
	a();
	b();
	printf("%s!\n", STRING);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . OS=foo >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
build/foo/xxx > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
In function a():  FOO.
In function b():  FOO.
FOO!
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog . OS=bar > cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
build/bar/xxx > bar.out
if test $? -ne 0 ; then fail 2 ; fi
diff - bar.out <<'_EOF_'
Only when BAR:
In function a():  BAR.
In function b():  BAR.
BAR!
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
