#!/bin/sh
#
#	Compile three separate executables from separate .c files
#	in the local directory, and two separate executables from
#	separate .c files in a subdirectory.  Invoke cons with no
#	arguments and make sure nothing gets built.  Supply one of
#	the local executables as an argument and make sure only it
#	gets built.  Supply one of the local executables and the
#	subdirectory as arguments and make sure only those executables
#	get built.  Supply '.' as an argument and make sure the
#	last local executable gets built.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [selective targeting]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir src
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'aaa', 'aaa.c';
Program \$env 'bbb', 'bbb.c';
Program \$env 'ccc', 'ccc.c';
Build qw (
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >aaa.c
main()
{
	printf("This is the aaa.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >bbb.c
main()
{
	printf("This is the bbb.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >ccc.c
main()
{
	printf("This is the ccc.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >src/Conscript		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'xxx', 'xxx.c';
Program \$env 'yyy', 'yyy.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/xxx.c
main()
{
	printf("This is the src/xxx.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/yyy.c
main()
{
	printf("This is the src/yyy.c file.\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
if test -x aaa ; then fail 1 ; fi
if test -x bbb ; then fail 1 ; fi
if test -x ccc ; then fail 1 ; fi
if test -x src/xxx ; then fail 1 ; fi
if test -x src/yyy ; then fail 1 ; fi

$prog aaa > cons.2.out 2>&1
if test -x bbb ; then fail 2 cons.2.out ; fi
if test -x ccc ; then fail 2 ; fi
if test -x src/xxx ; then fail 2 ; fi
if test -x src/yyy ; then fail 2 ; fi
./aaa > aaa.out
if test $? -ne 0 ; then fail 2 ; fi
diff - aaa.out <<'_EOF_'
This is the aaa.c file.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

$prog bbb src > cons.3.out 2>&1
if test -x ccc ; then fail 3 cons.3.out ; fi
./bbb > bbb.out
if test $? -ne 0 ; then fail 3 ; fi
diff - bbb.out <<'_EOF_'
This is the bbb.c file.
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
src/xxx > xxx.out
if test $? -ne 0 ; then fail 3 ; fi
diff - xxx.out <<'_EOF_'
This is the src/xxx.c file.
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
src/yyy > yyy.out
if test $? -ne 0 ; then fail 3 ; fi
diff - yyy.out <<'_EOF_'
This is the src/yyy.c file.
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

$prog . > cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
./ccc > ccc.out
if test $? -ne 0 ; then fail 4 ; fi
diff - ccc.out <<'_EOF_'
This is the ccc.c file.
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

#
pass
