#!/bin/sh
#
#	Define an executable to be built from a single .c file in
#	the local directory.  The .c files prints a single string
#	to be supplied via '-DSTRING=' from CFLAGS in the build
#	environment.  Create an override file that changes CFLAGS
#	when generating the .o file.  Invoke cons; make sure the
#	executable is built with the default STRING value.  Invoke
#	cons with the override file; make sure the executable is
#	built with the overridden STRING value.  Invoke cons again,
#	without the override; make sure the executable has reverted
#	to the original STRING value.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-o overrides]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons (
	CC	=> '$CC',
	COPT	=> '-DSTRING=\"FOO\"',
	CFLAGS	=> '%COPT',
);
Program \$env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo.c
main()
{
	printf("This is the foo.c file:  %s.\n", STRING);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >over
Override '\.o$', COPT => '-DSTRING=\"OVERRIDE\"';
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
./foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
This is the foo.c file:  FOO.
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog -o over . > cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
./foo > foo.out
if test $? -ne 0 ; then fail 2 ; fi
diff - foo.out <<'_EOF_'
This is the foo.c file:  OVERRIDE.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

$prog . > cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
./foo > foo.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.out <<'_EOF_'
This is the foo.c file:  FOO.
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

#
pass
