#!/bin/sh
#
#	Create a work subdirectory and a repository subdirectory.
#	The repository contains four .c files to build into an
#	executable and a Construct file.  Invoke cons -R in the
#	work subdirectory; check that the executable was built
#	correctly from repository .c files.  Create a work subdirectory
#	copy of one of the .c files; invoke cons -R; check that
#	the executable uses the one work .c files and the rest from
#	the repository.  Create work copies of the remaining .c
#	files; cons -R; check that the executable uses all the work
#	.c files.  Remove one work .c files; cons -R; check that
#	the executable uses the repository .c file for the one
#	removed.  Remove the rest of the work .c files; cons -R;
#	check that the executable uses all the repository .c files.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [multi-module Program, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir repository
if test $? -ne 0 ; then exit 2 ; fi
mkdir work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', qw (
	aaa.c
	bbb.c
	ccc.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/aaa.c
aaa()
{
	printf("repository/aaa.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bbb.c
bbb()
{
	printf("repository/bbb.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/ccc.c
ccc()
{
	printf("repository/ccc.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/main.c
main()
{
	aaa();
	bbb();
	ccc();
	printf("repository/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
repository/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

cat <<'_EOF_' >work/bbb.c
bbb()
{
	printf("work/bbb.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
work/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.err ; fi
diff - cons.4.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
diff - cons.4.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

cat <<'_EOF_' >work/aaa.c
aaa()
{
	printf("work/aaa.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/ccc.c
ccc()
{
	printf("work/ccc.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/main.c
main()
{
	aaa();
	bbb();
	ccc();
	printf("work/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 5 ; fi
diff - foo.out <<'_EOF_'
work/aaa.c
work/bbb.c
work/ccc.c
work/main.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.6.out 2>cons.6.err
if test $? -ne 0 ; then fail 6 cons.6.err ; fi
diff - cons.6.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 6 ; fi
diff - cons.6.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 6 ; fi

rm -f work/ccc.c
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.7.out 2>&1
if test $? -ne 0 ; then fail 7 cons.7.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 7 ; fi
diff - foo.out <<'_EOF_'
work/aaa.c
work/bbb.c
repository/ccc.c
work/main.c
_EOF_
if test $? -ne 0 ; then fail 7 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.8.out 2>cons.8.err
if test $? -ne 0 ; then fail 8 cons.8.err ; fi
diff - cons.8.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 8 ; fi
diff - cons.8.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 8 ; fi

rm -f work/aaa.c
if test $? -ne 0 ; then no_result ; fi
rm -f work/bbb.c
if test $? -ne 0 ; then no_result ; fi
rm -f work/main.c
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.9.out 2>&1
if test $? -ne 0 ; then fail 9 cons.9.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 9 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
repository/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 9 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.10.out 2>cons.10.err
if test $? -ne 0 ; then fail 10 cons.10.err ; fi
diff - cons.10.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 10 ; fi
diff - cons.10.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 10 ; fi

#
pass
