#!/bin/sh
#
#	Create a work subdirectory and a repository subdirectory.
#	The repository contains one .c file, which includes a .h
#	file, which in turn includes another .h file.  The work
#	subdirectory contains a Construct file with Repository
#	pointing to the repository and 'Program' to build an
#	executable from the .c file.  Invoke cons in the work
#	subdirectory to build the executable from the repository
#	source files.  Check that the executable was built correctly.
#	Create a work copy of the "intermediate" .h file.  Build
#	again.  Check that the executable picked up the work .h
#	file.  Create a work copy of the .c file.  Build again and
#	check the executable.  Create a work copy of the last .h
#	file.  Build and check...  Remove the work copies of the
#	.c file and the intermediate .h file.  Build and check...
#	Remove the work copy of the last .h file.  Build and check
#	that we're back to the repository copies of all files.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Repository nested .h files]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir repository
if test $? -ne 0 ; then exit 2 ; fi
mkdir work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES, EXPAND $CC, $work
\$env = new cons (
	CC	=> '$CC',
	CPPPATH => '.',
);
Repository '$work/repository';
Program \$env 'foo', 'main.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi

cat <<'_EOF_' >repository/foo.h
#define	STRING1	"repository/foo.h"
#include <bar.h>
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bar.h
#define	STRING2	"repository/bar.h"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/main.c
#include <foo.h>
main()
{
	printf("%s\n", STRING1);
	printf("%s\n", STRING2);
	printf("repository/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
work/foo > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/foo.h
repository/bar.h
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

cat <<'_EOF_' >work/foo.h
#define	STRING1	"work/foo.h"
#include <bar.h>
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/foo > foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
work/foo.h
repository/bar.h
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd work && $prog .) >cons.4.out 2>cons.4.err
if test $? -ne 0 ; then fail 4 cons.4.err ; fi
diff - cons.4.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
diff - cons.4.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

cat <<'_EOF_' >work/main.c
#include <foo.h>
main()
{
	printf("%s\n", STRING1);
	printf("%s\n", STRING2);
	printf("work/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
work/foo > foo.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - foo.5.out <<'_EOF_'
work/foo.h
repository/bar.h
work/main.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

(set +x; cd work && $prog .) >cons.6.out 2>cons.6.err
if test $? -ne 0 ; then fail 6 cons.6.err ; fi
diff - cons.6.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 6 ; fi
diff - cons.6.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 6 ; fi

cat <<'_EOF_' >work/bar.h
#define	STRING2	"work/bar.h"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.7.out 2>&1
if test $? -ne 0 ; then fail 7 cons.7.out ; fi
work/foo > foo.7.out
if test $? -ne 0 ; then fail 7 ; fi
diff - foo.7.out <<'_EOF_'
work/foo.h
work/bar.h
work/main.c
_EOF_
if test $? -ne 0 ; then fail 7 ; fi

(set +x; cd work && $prog .) >cons.8.out 2>cons.8.err
if test $? -ne 0 ; then fail 8 cons.8.err ; fi
diff - cons.8.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 8 ; fi
diff - cons.8.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 8 ; fi

rm -f work/main.c
if test $? -ne 0 ; then no_result ; fi
rm -f work/foo.h
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.9.out 2>&1
if test $? -ne 0 ; then fail 9 cons.9.out ; fi
work/foo > foo.9.out
if test $? -ne 0 ; then fail 9 ; fi
diff - foo.9.out <<'_EOF_'
repository/foo.h
work/bar.h
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 9 ; fi

(set +x; cd work && $prog .) >cons.10.out 2>cons.10.err
if test $? -ne 0 ; then fail 10 cons.10.err ; fi
diff - cons.10.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 10 ; fi
diff - cons.10.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 10 ; fi

rm -f work/bar.h
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.11.out 2>&1
if test $? -ne 0 ; then fail 11 cons.11.out ; fi
work/foo > foo.11.out
if test $? -ne 0 ; then fail 11 ; fi
diff - foo.11.out <<'_EOF_'
repository/foo.h
repository/bar.h
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 11 ; fi

(set +x; cd work && $prog .) >cons.12.out 2>cons.12.err
if test $? -ne 0 ; then fail 12 cons.12.err ; fi
diff - cons.12.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 12 ; fi
diff - cons.12.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 12 ; fi

#
pass
