#!/bin/sh
#
#	Create a repository directory with four .c files and a
#	Construct file pointing to the Repository.  Build the
#	executable in the repository directory.  "Lock" the repository
#	directory by removing all write permissions.  Copy the
#	Construct file to the work directory.  Invoke cons in the
#	work subdirectory.  check that it didn't re-build the
#	executable (the repository binary is up-to-date).  Create
#	a work .c file; invoke cons in the work subdirectory; check
#	that the executable re-built in the work subdirectory with
#	the work .c file.  Remove the work .c file; cons; check
#	that the executable re-built in the work directory with
#	the repository .c file again.  Re-build in the repository
#	to check that nothing there re-builds.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Repository link with .o from repository]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir repository
if test $? -ne 0 ; then exit 2 ; fi
mkdir work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct	# NO SINGLE QUOTES, EXPAND $CC, $work
\$env = new cons ( CC => '$CC' );
Repository '$work/repository';
Program \$env 'foo', qw (
	aaa.c
	bbb.c
	ccc.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/aaa.c
aaa()
{
	printf("repository/aaa.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bbb.c
bbb()
{
	printf("repository/bbb.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/ccc.c
ccc()
{
	printf("repository/ccc.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/main.c
main()
{
	aaa();
	bbb();
	ccc();
	printf("repository/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
repository/foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
repository/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd repository; $prog .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

# "Check out" the Construct file.
cp repository/Construct work/Construct
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.3.out 2>cons.3.err
if test $? -ne 0 ; then fail 3 cons.3.err ; fi
diff - cons.3.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
diff - cons.3.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

cat <<'_EOF_' >work/bbb.c
bbb()
{
	printf("work/bbb.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 4 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
work/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
if test -s work/aaa.o ; then fail 4 ; fi
if test -s work/ccc.o ; then fail 4 ; fi
if test -s work/main.o ; then fail 4 ; fi

rm -f work/bbb.c
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog .) >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
work/foo > foo.out
if test $? -ne 0 ; then fail 5 ; fi
diff - foo.out <<'_EOF_'
repository/aaa.c
repository/bbb.c
repository/ccc.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

# Make the repository writable again.
find repository -user $USER -exec chmod u+w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) >cons.6.out 2>cons.6.err
if test $? -ne 0 ; then fail 6 cons.6.out ; fi
diff - cons.6.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 6 ; fi
diff - cons.6.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 6 ; fi

#
pass
