#!/bin/sh
#
#	In the repository, define a single executable compiled from
#	three .c files in a source subdirectory.  One module has
#	a conditional #include for the selected OS.  Compilation
#	takes place in two separate build subdirectories established
#	via the 'Link' command in two separate invocations of cons.
#	The appropriate build directory is specified via "OS=" on
#	the cons command line.  Build one OS in the repository.
#	Invoke cons for the same OS in the work subdirectory;
#	nothing should get built.  Invoke cons for the second OS
#	in the work subdirectory, check that it built properly.
#	Update a .h file in the work subdirectory and re-build the
#	second OS; make sure it picked up the new .h file.  Invoke
#	cons for the first OS in the work subdirectory; check that
#	it built correctly.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [single-environment variant builds, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct		# NO SINGLE QUOTES, EXPAND $CC
die qq(OS must be specified) unless \$OS = \$ARG{OS};
\$BUILD = "#build/\$OS";
%cflags = (
	'foo'	=> '-DFOO',
	'bar'	=> '-DBAR',
);
\$env = new cons (
	CC	=> '$CC',
	CFLAGS => \$cflags{\$OS},
);
Export ( env );
Link \$BUILD => 'src';
Build (
	"\$BUILD/Conscript"
);
_EOF_
cat <<'_EOF_' >repository/src/Conscript
Import ( env );
$my_env = $env->clone(
	CPPPATH => 'src',
);
Program $my_env 'xxx', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/string.h
#ifdef	FOO
#define	STRING	"REPOSITORY_FOO"
#endif
#ifdef	BAR
#define	STRING	"REPOSITORY_BAR"
#endif
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/a.c
#include <string.h>
a()
{
	printf("repository/src/a.c:  %s\n", STRING);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/b.c
#include <string.h>
b()
{
	printf("repository/src/b.c:  %s\n", STRING);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/main.c
#include <string.h>
main()
{
#ifdef	BAR
	printf("Only when #define BAR\n");
#endif
	a();
	b();
	printf("repository/src/main.c:  %s\n", STRING);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog . OS=foo) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
repository/build/foo/xxx > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
repository/src/a.c:  REPOSITORY_FOO
repository/src/b.c:  REPOSITORY_FOO
repository/src/main.c:  REPOSITORY_FOO
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository . OS=foo) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -s work/build/foo/a.o ; then fail 2 ; fi
if test -s work/build/foo/b.o ; then fail 2 ; fi
if test -s work/build/foo/main.o ; then fail 2 ; fi
if test -s work/build/foo/xxx ; then fail 2 ; fi

(set +x; cd work && $prog -R $work/repository . OS=bar) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/build/bar/xxx > bar.out
if test $? -ne 0 ; then fail 3 ; fi
diff - bar.out <<'_EOF_'
Only when #define BAR
repository/src/a.c:  REPOSITORY_BAR
repository/src/b.c:  REPOSITORY_BAR
repository/src/main.c:  REPOSITORY_BAR
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

cat <<'_EOF_' >work/src/string.h
#ifdef	FOO
#define	STRING	"WORK_FOO"
#endif
#ifdef	BAR
#define	STRING	"WORK_BAR"
#endif
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository . OS=bar) >cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/build/bar/xxx > bar.out
if test $? -ne 0 ; then fail 4 ; fi
diff - bar.out <<'_EOF_'
Only when #define BAR
repository/src/a.c:  WORK_BAR
repository/src/b.c:  WORK_BAR
repository/src/main.c:  WORK_BAR
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

(set +x; cd work && $prog -R $work/repository . OS=foo) >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
work/build/foo/xxx > bar.out
if test $? -ne 0 ; then fail 5 ; fi
diff - bar.out <<'_EOF_'
repository/src/a.c:  WORK_FOO
repository/src/b.c:  WORK_FOO
repository/src/main.c:  WORK_FOO
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

#
pass
