#!/bin/sh
#
#	In a repository, create xxx and include subdirectories in
#	a src subdirectory, and a build subdirectory with separate
#	subdirectores for two variant "OS" builds.  CPPPATH in the
#	repository Construct file specifies src/xxx:src/include.
#	A .c file in the xxx subdirectory #includes a .h file in
#	the xxx subdirectory, which nested #includes a .h file in
#	the include subdirectory.  Build in the repository.  Build
#	in the work directory; everything should still be up-to-date.
#	Create a work copy of the include subdirectory .h file;
#	build.  Create a work copy of the xxx subdirectory .h file;
#	build.  Remove the work copy of the include subdirectory
#	.h file; build.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [multi-environment variant builds, -R, multi-dir .h]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi
mkdir work/src/include
if test $? -ne 0 ; then no_result ; fi
mkdir work/src/xxx
if test $? -ne 0 ; then no_result ; fi
mkdir work/build
if test $? -ne 0 ; then no_result ; fi
mkdir work/build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir work/build/bar
if test $? -ne 0 ; then no_result ; fi

mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src/include
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src/xxx
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build/foo
if test $? -ne 0 ; then no_result ; fi
mkdir repository/build/bar
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<'_EOF_' >repository/Construct
Build qw(
	build/foo/Conscript
	build/bar/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/build/foo/Conscript	# NO SINGLE QUOTES, EXPAND $CC
Link 'src' => '#src';

\$env = new cons (
	CC	=> '$CC',
	CFLAGS	=> '-DFOO',
	CPPPATH => 'src/xxx:src/include',
);

Export qw( env );

Build qw(
	src/xxx/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/build/bar/Conscript	# NO SINGLE QUOTES, EXPAND $CC
Link 'src' => '#src';

\$env = new cons (
	CC	=> '$CC',
	CFLAGS	=> '-DBAR',
	CPPPATH => 'src/xxx:src/include',
);

Export qw( env );

Build qw(
	src/xxx/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/xxx/Conscript
Import qw( env );
Program $env 'xxx', qw (
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/include/my_string.h
#ifdef	FOO
#define	INCLUDE_OS	"FOO"
#endif
#ifdef	BAR
#define	INCLUDE_OS	"BAR"
#endif
#define	INCLUDE_STRING	"repository/src/include/my_string.h:  %s\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/xxx/include.h
#include <my_string.h>
#ifdef	FOO
#define	XXX_OS		"FOO"
#endif
#ifdef	BAR
#define	XXX_OS		"BAR"
#endif
#define	XXX_STRING	"repository/src/xxx/include.h:  %s\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/xxx/main.c
#include <include.h>
#ifdef	FOO
#define	MAIN_OS		"FOO"
#endif
#ifdef	BAR
#define	MAIN_OS		"BAR"
#endif
main()
{
	printf(INCLUDE_STRING, INCLUDE_OS);
	printf(XXX_STRING, XXX_OS);
	printf("repository/src/xxx/main.c:  %s\n", MAIN_OS);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository; $prog .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 ; fi
repository/build/foo/src/xxx/xxx > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/src/include/my_string.h:  FOO
repository/src/xxx/include.h:  FOO
repository/src/xxx/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
repository/build/bar/src/xxx/xxx > bar.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - bar.1.out <<'_EOF_'
repository/src/include/my_string.h:  BAR
repository/src/xxx/include.h:  BAR
repository/src/xxx/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -s work/build/foo/src/a.o ; then fail 2 ; fi
if test -s work/build/foo/src/b.o ; then fail 2 ; fi
if test -s work/build/foo/src/main.o ; then fail 2 ; fi
if test -s work/build/foo/src/xxx ; then fail 2 ; fi
if test -s work/build/bar/src/a.o ; then fail 2 ; fi
if test -s work/build/bar/src/b.o ; then fail 2 ; fi
if test -s work/build/bar/src/main.o ; then fail 2 ; fi
if test -s work/build/bar/src/xxx ; then fail 2 ; fi

sleep 1

cat <<'_EOF_' >work/src/include/my_string.h
#ifdef	FOO
#define	INCLUDE_OS	"FOO"
#endif
#ifdef	BAR
#define	INCLUDE_OS	"BAR"
#endif
#define	INCLUDE_STRING	"work/src/include/my_string.h:  %s\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/build/foo/src/xxx/xxx > foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
work/src/include/my_string.h:  FOO
repository/src/xxx/include.h:  FOO
repository/src/xxx/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/build/bar/src/xxx/xxx > bar.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - bar.3.out <<'_EOF_'
work/src/include/my_string.h:  BAR
repository/src/xxx/include.h:  BAR
repository/src/xxx/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

sleep 1

cat <<'_EOF_' >work/src/xxx/include.h
#include <my_string.h>
#ifdef	FOO
#define	XXX_OS		"FOO"
#endif
#ifdef	BAR
#define	XXX_OS		"BAR"
#endif
#define	XXX_STRING	"work/src/xxx/include.h:  %s\n"
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/build/foo/src/xxx/xxx > foo.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - foo.4.out <<'_EOF_'
work/src/include/my_string.h:  FOO
work/src/xxx/include.h:  FOO
repository/src/xxx/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 4 ; fi
work/build/bar/src/xxx/xxx > bar.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - bar.4.out <<'_EOF_'
work/src/include/my_string.h:  BAR
work/src/xxx/include.h:  BAR
repository/src/xxx/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

rm -f work/src/include/my_string.h
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.5.out 2>&1
if test $? -ne 0 ; then fail 5 cons.5.out ; fi
work/build/foo/src/xxx/xxx > foo.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - foo.5.out <<'_EOF_'
repository/src/include/my_string.h:  FOO
work/src/xxx/include.h:  FOO
repository/src/xxx/main.c:  FOO
_EOF_
if test $? -ne 0 ; then fail 5 ; fi
work/build/bar/src/xxx/xxx > bar.5.out
if test $? -ne 0 ; then fail 5 ; fi
diff - bar.5.out <<'_EOF_'
repository/src/include/my_string.h:  BAR
work/src/xxx/include.h:  BAR
repository/src/xxx/main.c:  BAR
_EOF_
if test $? -ne 0 ; then fail 5 ; fi

#
pass
