#!/bin/sh
#
#	In the repository subdirectory, create three .c files in
#	the local directory, and two .c files in a subdirectory,
#	with Construct and Conscript files to create separate
#	executables from each.  In the work subdirectory, invoke
#	cons with no arguments and make sure nothing gets built.
#	In the work subdirectory, invoke cons -R with one of the
#	local executables as an argument and make sure only it gets
#	built.  In the work subdirectory, invoke cons -R with one
#	of the local executables and the subdirectory as arguments
#	and make sure only those executables get built.  In the
#	work subdirectory, invoke cons -R with '.' as an argument
#	and make sure the last local executable gets built.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [selective targeting, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'aaa', 'aaa.c';
Program \$env 'bbb', 'bbb.c';
Program \$env 'ccc', 'ccc.c';
Build qw (
	src/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/aaa.c
main()
{
	printf("repository/aaa.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bbb.c
main()
{
	printf("repository/bbb.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/ccc.c
main()
{
	printf("repository/ccc.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/src/Conscript		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'xxx', 'xxx.c';
Program \$env 'yyy', 'yyy.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/xxx.c
main()
{
	printf("repository/src/xxx.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/yyy.c
main()
{
	printf("repository/src/yyy.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
if test -x aaa ; then fail 1 ; fi
if test -x bbb ; then fail 1 ; fi
if test -x ccc ; then fail 1 ; fi
if test -x src/xxx ; then fail 1 ; fi
if test -x src/yyy ; then fail 1 ; fi

(set +x; cd work && $prog -R $work/repository aaa) >cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.1.out ; fi
if test -x bbb ; then fail 2 ; fi
if test -x ccc ; then fail 2 ; fi
if test -x src/xxx ; then fail 2 ; fi
if test -x src/yyy ; then fail 2 ; fi
work/aaa > aaa.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - aaa.2.out <<'_EOF_'
repository/aaa.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

(set +x; cd work && $prog -R $work/repository bbb src) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
if test -x work/ccc ; then fail 3 ; fi
work/bbb > bbb.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - bbb.3.out <<'_EOF_'
repository/bbb.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/src/xxx > xxx.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - xxx.3.out <<'_EOF_'
repository/src/xxx.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
work/src/yyy > yyy.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - yyy.3.out <<'_EOF_'
repository/src/yyy.c
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/ccc > ccc.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - ccc.4.out <<'_EOF_'
repository/ccc.c
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

#
pass
