#!/bin/sh
#
#	Define three separate executables to be compiled from
#	separate .c files in the repository subdirectory.  Specify
#	a 'Default' target of one of the executables.  Invoke cons
#	in the work subdirectory with no arguments.  Make sure only
#	the default executable gets built.  Invoke cons in the work
#	subdirectory with '.' as argument.  Make sure the rest of
#	the executables get built.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Default, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'aaa', 'aaa.c';
Program \$env 'bbb', 'bbb.c';
Program \$env 'ccc', 'ccc.c';
Default qw(
	bbb
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/aaa.c
main()
{
	printf("repository/aaa.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bbb.c
main()
{
	printf("repository/bbb.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/ccc.c
main()
{
	printf("repository/ccc.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
if test -x work/aaa ; then fail 1 ; fi
if test -x work/ccc ; then fail 1 ; fi
work/bbb > bbb.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - bbb.1.out <<'_EOF_'
repository/bbb.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog -R $work/repository .) > cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
work/aaa > aaa.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - aaa.2.out <<'_EOF_'
repository/aaa.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
work/ccc > ccc.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - ccc.2.out <<'_EOF_'
repository/ccc.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
