#!/bin/sh
#
#	In the repository subdirectory, define two separate
#	executables to be compiled from single .c files via separate
#	Conscript files in separate subdirectories.  Invoke cons
#	in the work subdirectory, pruning the build to one of the
#	subdirectories ('+subdir') and make sure only the executable
#	in that subdirectory gets built.  Invoke cons in the work
#	subdirectory with '.' as an argument.  Make sure the other
#	subdirectory's executable now got built, too.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [build pruning, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/foo
if test $? -ne 0 ; then no_result ; fi
mkdir repository/bar
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<'_EOF_' >repository/Construct
Build qw (
	foo/Conscript
	bar/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/foo/Conscript		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/foo/foo.c
main()
{
	printf("repository/foo/foo.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/bar/Conscript		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'bar', 'bar.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/bar/bar.c
main()
{
	printf("repository/bar/bar.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository . +bar) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
if test -x work/foo/foo ; then fail 1 ; fi
work/bar/bar > bar.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - bar.1.out <<'_EOF_'
repository/bar/bar.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi

work/foo/foo > foo.2.out
if test $? -ne 0 ; then fail 2 ; fi
diff - foo.2.out <<'_EOF_'
repository/foo/foo.c
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
