#!/bin/sh
#
#	In a repository subdirectory, create Construct and Conscript
#	files for creating four executables, two from .c files in
#	the local directory and two from .c files in a subdirectory.
#	Invoke cons -pa -R in the work subdirectory.  compare the
#	output against expected results.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-pa, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/subdir
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'a', 'a.c';
Program \$env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/a.c
main()
{
	printf("repository/a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/b.c
main()
{
	printf("repository/b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/subdir/Conscript	# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'c', 'c.c';
Program \$env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/c.c
main()
{
	printf("repository/subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/d.c
main()
{
	printf("repository/subdir/d.c\n");
	exit (0);
}
_EOF_

(set +x; cd work && $prog -R $work/repository -pa .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - cons.1.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $CC, $work
a:
... $CC -o a a.o
a.o:
... $CC -c $work/repository/a.c -o a.o
b:
... $CC -o b b.o
b.o:
... $CC -c $work/repository/b.c -o b.o
subdir/c:
... $CC -o subdir/c subdir/c.o
subdir/c.o:
... $CC -c $work/repository/subdir/c.c -o subdir/c.o
subdir/d:
... $CC -o subdir/d subdir/d.o
subdir/d.o:
... $CC -c $work/repository/subdir/d.c -o subdir/d.o
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
if test -s repository/a ; then fail 1 ; fi
if test -s repository/b ; then fail 1 ; fi
if test -s repository/subdir/c ; then fail 1 ; fi
if test -s repository/subdir/d ; then fail 1 ; fi

#
pass
