#!/bin/sh
#
#	In a repository subdirectory, create Construct and Conscript
#	files for creating four executables, two from .c files in
#	the local directory and two from .c files in a subdirectory.
#	Invoke cons -d -R in the work subdirectory.  Compare the
#	output against expected results.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-d, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/subdir
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'a', 'a.c';
Program \$env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/a.c
main()
{
	printf("repository/a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/b.c
main()
{
	printf("repository/b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/subdir/Conscript	# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'c', 'c.c';
Program \$env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/c.c
main()
{
	printf("repository/subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/subdir/d.c
main()
{
	printf("repository/subdir/d.c\n");
	exit (0);
}
_EOF_

(set +x; cd work && $prog -R $work/repository -d .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - cons.1.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $CC, $work
a
  a.o
    a.c
$CC -c $work/repository/a.c -o a.o
$CC -o a a.o
b
  b.o
    b.c
$CC -c $work/repository/b.c -o b.o
$CC -o b b.o
subdir/c
  subdir/c.o
    subdir/c.c
$CC -c $work/repository/subdir/c.c -o subdir/c.o
$CC -o subdir/c subdir/c.o
subdir/d
  subdir/d.o
    subdir/d.c
$CC -c $work/repository/subdir/d.c -o subdir/d.o
$CC -o subdir/d subdir/d.o
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/a >a.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - a.1.out <<'_EOF_'
repository/a.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/b >b.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - b.1.out <<'_EOF_'
repository/b.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/subdir/c >c.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - c.1.out <<'_EOF_'
repository/subdir/c.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
work/subdir/d >d.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - d.1.out <<'_EOF_'
repository/subdir/d.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
